program Databaza;

uses Memory,Consts,Drivers,Objects,Views,Menus,App,Dialogs,
  Editors,StdDlg,Validate,Count,MsgBox;

type
  PRecord=^TRecord;
  TRecord=record
    RecordNum:string[4];
    Title:string[4];
    SureName:string[20];
    LastName:string[20];
    Phone:string[20];
    BirthDate:string[8];
    Sex:Word;
    Notes:array[0..255] of Char;
  end;

  PRecordObj = ^TRecordObj;
  TRecordObj = object(TObject)
    TransferRecord: TRecord;
    constructor Load(var S: TStream);
    procedure Store(var S: TStream);
  end;

  PRecordWindow = ^TRecordWindow;
  TRecordWindow = object(TDialog)
    Counter: PCountView;
    constructor Init;
    constructor Load(var S: TStream);
    destructor Done; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Store(var S: TStream); virtual;
  end;

  TdBaseApp = object(TApplication)
    ClipboardWindow: PEditWindow;
    RecordWindow: PRecordWindow;
    constructor Init;
    destructor Done; virtual;
    procedure CancelRecord;
    procedure DoAboutBox;
    procedure EnterNewRecord;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure LoadDesktop;
    procedure NewWindow;
    procedure OpenRecordWindow;
    procedure OpenWindow;
    procedure SaveDesktop;
    procedure SaveRecordData;
    procedure ShowRecord(ARecordNum: Integer);
  end;

var
  RecordInfo: TRecord;
  RecordColl: PCollection;
  CurrentRecord: Integer;
  TempRecord: PRecordObj;

const
  RRecordObj: TStreamRec = (
     ObjType: 15000;
     VmtLink: Ofs(TypeOf(TRecordObj)^);
     Load:    @TRecordObj.Load;
     Store:   @TRecordObj.Store
  );
  RRecordWindow: TStreamRec = (
     ObjType: 15001;
     VmtLink: Ofs(TypeOf(TRecordWindow)^);
     Load:    @TRecordWindow.Load;
     Store:   @TRecordWindow.Store
  );

procedure dBaseStreamError(var S:TStream);far;
var
  ErrorMessage:String;
begin
  case S.Status of
    stError: ErrorMessage := 'Stream access error';
    stInitError: ErrorMessage := 'Cannot initialize stream';
    stReadError: ErrorMessage := 'Read beyond end of stream';
    stWriteError: ErrorMessage := 'Cannot expand stream';
    stGetError: ErrorMessage := 'Unregistered type read from stream';
    stPutError: ErrorMessage := 'Unregistered type written to stream';
    end;
  DoneVideo;
  PrintStr('Error: ' + ErrorMessage);
  Halt(Abs(S.Status));
end;

procedure LoadRecords;
var
  RecordFile: TBufStream;
begin
  RecordFile.Init('DATABAZA.DAT', stOpenRead, 1024);
  RecordColl := PCollection(RecordFile.Get);
  RecordFile.Done;
end;

procedure SaveRecords;
var
  RecordFile: TBufStream;
begin
  RecordFile.Init('DATABAZA.DAT', stOpenWrite, 1024);
  RecordFile.Put(RecordColl);
  RecordFile.Done;
end;

constructor TRecordObj.Load(var S: TStream);
begin
  inherited Init;
  S.Read(TransferRecord, SizeOf(TransferRecord));
end;

procedure TRecordObj.Store(var S: TStream);
begin
  S.Write(TransferRecord, SizeOf(TransferRecord));
end;

constructor TRecordWindow.Init;
var
  R: TRect;
  Field: PInputLine;
  Cluster: PCluster;
  Memo: PMemo;
begin
  R.Assign(0, 0, 50, 17);
  inherited Init(R, 'Databza');
  Options := Options or ofCentered;
  HelpCtx := $F000;

{    RecordNum:string[4];     }
  R.Assign(18,2,47,3);
  Field:=New(PInputLine,Init(R,4));
  Field^.SetValidator(New(PRangeValidator,Init(1,9999)));
  Insert(Field);
  R.Assign(2,2,17,3);
  Insert(New(PLabel,Init(R,'Poradov ~~slo',Field)));
{    Title:string[4];        }
  R.Assign(9,3,47,4);
  Field:=New(PInputLine,Init(R,4));
  Field^.SetValidator(New(PPXPictureValidator,Init('{[@@@@]}',True)));
  Insert(Field);
  R.Assign(2,3,8,4);
  Insert(New(PLabel,Init(R,'~T~itul',Field)));
{    SureName:string[20];   }
  R.Assign(8,4,47,5);
  Field:=New(PInputLine,Init(R,20));
  Field^.SetValidator(New(PPXPictureValidator,Init('',True)));
  Insert(Field);
  R.Assign(2,4,7,5);
  Insert(New(PLabel,Init(R,'~M~eno',Field)));
{    LastName:string[20];  }
  R.Assign(14,5,47,6);
  Field:=New(PInputLine,Init(R,20));
  Field^.SetValidator(New(PPXPictureValidator,Init('',True)));
  Insert(Field);
  R.Assign(2,5,13,6);
  Insert(New(PLabel,Init(R,'~P~riezvisko',Field)));
{    Phone:string[20];    }
  R.Assign(11,6,47,7);
  Field:=New(PInputLine,Init(R,20));
  Field^.SetValidator(New(PPXPictureValidator,Init('', True)));
  Insert(Field);
  R.Assign(2,6,10,7);
  Insert(New(PLabel,Init(R,'Tele~f~n',Field)));
{    BirthDate:string[9];}
  R.Assign(19,7,47,8);
  Field:=New(PInputLine,Init(R,8));
  Field^.SetValidator(New(PPXPictureValidator,Init('{#[#]}/{#[#]}/{##[##]}', True)));
  Insert(Field);
  R.Assign(2,7,18,8);
  Insert(New(PLabel,Init(R,'Dtum ~n~arodenia',Field)));
{    Sex:Word;}
  R.Assign(12,8,47,9);
  Cluster:=New(PRadioButtons,Init(R,
    NewSItem('Mu ',NewSItem('ena',nil))));
  Insert(Cluster);
  R.Assign(2,8,11,9);
  Insert(New(PLabel,Init(R,'P~o~hlavie', Cluster)));
{    Notes:array[0..255] of Char;}
  R.Assign(3,10,47,13);
  Memo:=New(PMemo,Init(R, nil, nil, nil, 255));
  Insert(Memo);
  R.Assign(2,9,11,10);
  Insert(New(PLabel,Init(R,'Po~z~nmky',Memo)));


  R.Assign(2,14,10,16);
  Insert(New(PButton, Init(R, '~N~ov', cmRecordNew, bfNormal)));
  R.Assign(10,14,18,16);
  Insert(New(PButton, Init(R, '~U~lo', cmRecordSave, bfDefault)));
  R.Assign(18,14,27,16);
  Insert(New(PButton, Init(R, '~O~tvor', cmRecordCancel, bfNormal)));
  R.Assign(27,14,36,16);
  Insert(New(PButton, Init(R, 'Dal', cmRecordNext, bfNormal)));
  R.Assign(36,14,48,16);
  Insert(New(PButton, Init(R, 'Predol', cmRecordPrev, bfNormal)));

  R.Assign(5, 16, 20, 17);
  Counter := New(PCountView, Init(R));
  Counter^.SetCount(RecordColl^.Count);
  Insert(Counter);

  SelectNext(False);
end;

constructor TRecordWindow.Load(var S: TStream);
begin
  inherited Load(S);
  GetSubViewPtr(S, Counter);
end;

destructor TRecordWindow.Done;
begin
  DisableCommands([cmRecordNext, cmRecordPrev, cmRecordSave]);
  inherited Done;
end;

procedure TRecordWindow.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  if (Event.What = evBroadcast) and
    (Event.Command = cmFindRecordWindow) then
    ClearEvent(Event);
end;

procedure TRecordWindow.Store(var S: TStream);
begin
  inherited Store(S);
  PutSubViewPtr(S, Counter);
end;

constructor TdBaseApp.Init;
var
  R: TRect;
begin
  MaxHeapSize := 8192;
  EditorDialog := StdEditorDialog;
  StreamError := @dBaseStreamError;
  RegisterMenus;
  RegisterObjects;
  RegisterViews;
  RegisterApp;
  RegisterEditors;
  RegisterDialogs;
  RegisterValidate;
  RegisterType(RRecordObj);
  RegisterType(RRecordWindow);
  RegisterCount;
  inherited Init;
  Desktop^.GetExtent(R);
  ClipboardWindow := New(PEditWindow, Init(R, '', wnNoNumber));
  if ValidView(ClipboardWindow) <> nil then
  begin
    ClipboardWindow^.Hide;
    InsertWindow(ClipboardWindow);
    Clipboard := ClipboardWindow^.Editor;
    Clipboard^.CanUndo := False;
  end;
  LoadRecords;
  CurrentRecord := 0;
  RecordInfo := PRecordObj(RecordColl^.At(CurrentRecord))^.TransferRecord;
  DisableCommands([cmRecordNext, cmRecordPrev, cmRecordCancel, cmRecordSave]);
end;

destructor TdBaseApp.Done;
begin
  inherited Done;
end;

procedure TdBaseApp.CancelRecord;
begin
  if CurrentRecord < RecordColl^.Count then
    ShowRecord(CurrentRecord)
  else
  begin
    Dispose(TempRecord, Done);
    ShowRecord(CurrentRecord - 1);
  end;
end;

procedure TdBaseApp.DoAboutBox;
begin
  MessageBox(#3'Databza DOS version TV1.1'#13 +
    #3'Programmed by Ondrej Jombk'#13#3'Copyright 1998 Condy software inc.',
    nil, mfInformation or mfOKButton);
end;

procedure TdBaseApp.EnterNewRecord;
begin
  OpenRecordWindow;
  CurrentRecord := RecordColl^.Count;
  TempRecord := New(PRecordObj, Init);
  RecordInfo := TempRecord^.TransferRecord;
  with RecordWindow^ do
  begin
    SetData(RecordInfo);
    Counter^.SetCurrent(CurrentRecord + 1);
  end;
  DisableCommands([cmRecordNext, cmRecordPrev, cmRecordNew]);
  EnableCommands([cmRecordCancel, cmRecordSave]);
end;

procedure TdBaseApp.HandleEvent(var Event: TEvent);
var
  R: TRect;
begin
  inherited HandleEvent(Event);
  if Event.What = evCommand then
  begin
    case Event.Command of
      cmRecordNew:
        begin
          EnterNewRecord;
          ClearEvent(Event);
        end;
      cmRecordCancel:
        begin
          CancelRecord;
          ClearEvent(Event);
        end;
      cmRecordNext:
        begin
          ShowRecord(CurrentRecord + 1);
          ClearEvent(Event);
        end;
      cmRecordPrev:
        begin
          ShowRecord(CurrentRecord - 1);
          ClearEvent(Event);
        end;
      cmRecordSave:
        begin
          SaveRecordData;
          ClearEvent(Event);
        end;
      cmRecordWin:
        begin
          OpenRecordWindow;
          ClearEvent(Event);
        end;
      cmOptionsLoad:
        begin
          LoadDesktop;
          ClearEvent(Event);
        end;
      cmOptionsSave:
        begin
          SaveDesktop;
          ClearEvent(Event);
        end;
      cmClipShow:
        with ClipboardWindow^ do
        begin
          Select;
          Show;
          ClearEvent(Event);
        end;
      cmNew:
        begin
          NewWindow;
          ClearEvent(Event);
        end;
      cmOpen:
        begin
          OpenWindow;
          ClearEvent(Event);
        end;
      cmOptionsVideo:
        begin
          SetScreenMode(ScreenMode xor smFont8x8);
          ClearEvent(Event);
        end;
      cmAbout:
        begin
          DoAboutBox;
          ClearEvent(Event);
        end;
    end;
  end;
end;

procedure TdBaseApp.InitMenuBar;
var
  R:TRect;
begin
  GetExtent(R);
  R.B.Y:=R.A.Y+1;
  MenuBar:=New(PMenuBar,Init(R,NewMenu(
    NewSubMenu('~S~bor',hcNoContext,NewMenu(
      NewItem('~N~ov','',kbNokey,cmNew,hcNew,
      NewItem('~O~tvori','F3',kbF3,cmOpen,hcOpen,
      NewItem('~U~lozi','F2',kbF2,cmSave,hcSave,
      NewItem('Ulozi ~a~ko...','Alt+F2',kbAltF2,cmSaveAs,hcSaveAs,
      NewItem('Ulozi ~v~etko','Ctrl+F2',kbCtrlF2,cmSaveAll,hcSaveAll,
      NewLine(
      NewItem('~Z~meni adresr','',kbNokey,cmChangeDir,hcChangeDir,
      NewItem('~S~kok do DOSu','',kbNokey,cmDosShell,hcDosShell,
      NewItem('~K~oniec','Alt+X',kbAltX,cmQuit,hcExit,
      nil)))))))))),
    NewSubMenu('~~pravy', hcNoContext, NewMenu(
      NewItem('~Z~pt','Alt+BackSpace',kbAltSpace,cmUndo,hcUndo,
      NewLine(
      NewItem('~V~ybra','Shift+Del',kbShiftDel,cmCut,hcCut,
      NewItem('~K~oprova','Ctrl+Ins',kbCtrlIns,cmCopy,hcCopy,
      NewItem('V~l~oi','Shift+Ins',kbShiftIns,cmPaste,hcPaste,
      NewItem('V~y~mazat','Ctrl+Del',kbCtrlDel,cmClear,hcClear,
      NewLine(
      NewItem('~O~bsah schrnky','',kbNoKey,cmClipShow,hcNoContext,
      nil))))))))),
    NewSubMenu('~D~atabza', hcNoContext, NewMenu(
      NewItem('~N~ov', 'F9', kbF9, cmRecordNew, hcNoContext,
      NewItem('~U~lo', '', kbNoKey, cmRecordSave, hcNoContext,
      NewLine(
      NewItem('Dal', 'PgDn', kbPgDn, cmRecordNext, hcNoContext,
      NewItem('Predol', 'PgUp', kbPgUp, cmRecordPrev, hcNoContext,
      nil)))))),
    NewSubMenu('~M~onosti',hcNoContext,NewMenu(
      NewItem('~Z~me video md','',kbNoKey,cmOptionsVideo,hcNoContext,
      NewItem('~U~lo desktop','',kbNoKey,cmOptionsSave,hcNoContext,
      NewItem('~N~ahraj desktop','',kbNoKey,cmOptionsLoad,hcNoContext,
      nil)))),
    NewSubMenu('~O~kno',hcNoContext,NewMenu(
      NewItem('~D~atabza','',kbNoKey,cmRecordWin,hcNoContext,
      NewLine(
      NewItem('~D~ladice','',kbNoKey,cmTile,hcTile,
      NewItem('~K~askda','',kbNoKey,cmCascade,hcCascade,
      NewItem('Zavrie ~v~etko','',kbNoKey,cmCloseAll,hcCloseAll,
      NewLine(
      NewItem('~P~resun','Ctrl+F5',kbCtrlF5,cmResize,hcResize,
      NewItem('Ve~~kos','F5',kbF5,cmZoom,hcZoom,
      NewItem('Dalie','F6',kbF6,cmNext,hcNext,
      NewItem('Predol','Shift+F6',kbShiftF6,cmPrev,hcPrev,
      NewItem('~Z~avrie','Alt+F3',kbAltF3,cmClose,hcClose,
      nil)))))))))))),
    NewSubMenu('~P~omoc', hcNoContext, NewMenu(
      NewItem('~I~nformcie', '', kbNoKey, cmAbout, hcNoContext,
      nil)),
    nil))))))
  )));
end;

procedure TdBaseApp.InitStatusLine;
var
  R:TRect;
begin
  GetExtent(R);
  R.A.Y:=R.B.Y-1;
  New(StatusLine,Init(R,
    NewStatusDef(0,$EFFF,
      NewStatusKey('~F3~ Otvor',kbF3,cmOpen,
      NewStatusKey('~F4~ Nov',kbF4,cmNew,
      NewStatusKey('~Alt+F3~ Zatvor',kbAltF3,cmClose,
      nil))),
    NewStatusDef($F000, $FFFF,
      NewStatusKey('~F6~ Dal',kbF6,cmRecordNext,
      NewStatusKey('~Shift+F6~ Predol',kbShiftF6,cmRecordPrev,
      nil)),
    nil))));
end;

procedure TdBaseApp.LoadDesktop;
var
  DesktopFile: TBufStream;
  TempDesktop: PDesktop;
  R: TRect;
begin
  DesktopFile.Init('DESKTOP.TUT', stOpenRead, 1024);
  TempDesktop := PDesktop(DesktopFile.Get);
  DesktopFile.Done;
  if ValidView(TempDesktop) <> nil then
  begin
    Desktop^.Delete(ClipboardWindow);
    Delete(Desktop);
    Dispose(Desktop, Done);
    Desktop := TempDesktop;
    Insert(Desktop);
    GetExtent(R);
    R.Grow(0, -1);
    Desktop^.Locate(R);
    InsertWindow(ClipboardWindow);
    RecordWindow := Message(Desktop, evBroadcast, cmFindRecordWindow, nil);
    if RecordWindow <> nil then ShowRecord(CurrentRecord);
  end;
end;

procedure TdBaseApp.NewWindow;
var
  R: TRect;
  TheWindow: PEditWindow;
begin
  R.Assign(0, 0, 60, 20);
  TheWindow := New(PEditWindow, Init(R, '', wnNoNumber));
  InsertWindow(TheWindow);
end;

procedure TdBaseApp.OpenRecordWindow;
begin
  if Message(Desktop, evBroadcast, cmFindRecordWindow, nil) = nil then
  begin
    RecordWindow := New(PRecordWindow, Init);
    InsertWindow(RecordWindow);
  end
  else
    if PView(RecordWindow) <> Desktop^.TopView then RecordWindow^.Select;
  ShowRecord(0);
end;

procedure TdBaseApp.OpenWindow;
var
  R: TRect;
  FileDialog: PFileDialog;
  TheFile: FNameStr;
const
  FDOptions: Word = fdOpenButton;
begin
  TheFile := '*.*';
  New(FileDialog, Init(TheFile, 'Otvorit subor', '~N~zov suboru',
    FDOptions, 1));
  if ExecuteDialog(FileDialog, @TheFile) <> cmCancel then
  begin
    R.Assign(0, 0, 75, 20);
    InsertWindow(New(PEditWindow, Init(R, TheFile, wnNoNumber)));
  end;
end;

procedure TdBaseApp.SaveDesktop;
var
  DesktopFile: TBufStream;
begin
  Desktop^.Delete(ClipboardWindow);
  DesktopFile.Init('DESKTOP.TUT', stCreate, 1024);
  DesktopFile.Put(Desktop);
  DesktopFile.Done;
  InsertWindow(ClipboardWindow);
end;

procedure TdBaseApp.SaveRecordData;
begin
  if RecordWindow^.Valid(cmClose) then
  begin
    RecordWindow^.GetData(RecordInfo);
    if CurrentRecord = RecordColl^.Count then
    begin
      TempRecord^.TransferRecord := RecordInfo;
      RecordColl^.Insert(TempRecord);
      RecordWindow^.Counter^.IncCount;
    end
    else PRecordObj(RecordColl^.At(CurrentRecord))^.TransferRecord := RecordInfo;
    SaveRecords;
    ShowRecord(CurrentRecord);
  end;
end;

procedure TdBaseApp.ShowRecord(ARecordNum: Integer);
begin
  CurrentRecord := ARecordNum;
  RecordInfo := PRecordObj(RecordColl^.At(CurrentRecord))^.TransferRecord;
  with RecordWindow^ do
  begin
    SetData(RecordInfo);
    Counter^.SetCurrent(CurrentRecord + 1);
  end;
  if CurrentRecord > 0 then EnableCommands([cmRecordPrev])
  else DisableCommands([cmRecordPrev]);
  if RecordColl^.Count > 0 then EnableCommands([cmRecordNext]);
  if CurrentRecord >= RecordColl^.Count - 1 then DisableCommands([cmRecordNext]);
  EnableCommands([cmRecordSave, cmRecordNew]);
end;

var
  dBaseApp: TdBaseApp;

begin
  dBaseApp.Init;
  dBaseApp.Run;
  dBaseApp.Done;
end.
